local AddonName, a = ...
if a.BuildFail(50000) then return end
local L = a.Localize
local s = SpellFlashAddon
local x = s.UpdatedVariables
local _

-- SpellFlash API:
-- http://wow.curseforge.com/addons/spellflash/pages/api/

-- Updated variable references:
-- http://wow.curseforge.com/addons/spellflash/pages/api/#w-local-x-spell-flash-addon-updated-variables
-- x.Enemy, x.ActiveEnemy, x.NoCC, x.InInstance, x.InstanceType, x.PetAlive, x.PetActiveEnemy, x.PetNoCC, x.Lag, x.DoubleLag, x.ThreatPercent, x.EnemyDetected, x.ShouldPermanentBuff, x.ShouldTemporaryBuff

-- Spell table references:
-- http://wow.curseforge.com/addons/spellflash/pages/api/#w-s-castable

-- Glyphs tested:
-- Everlasting Affliction
-- Dark Soul
-- Life Tap
-- Burning Embers
-- Conflagrate
-- Verdant Spheres


local SpellPowerMultiplier = {
	1459--[[Arcane Brilliance]],
	61316--[[Dalaran Brilliance]],
	77747--[[Burning Wrath]],
	109773--[[Dark Intent]],
	126309--[[Still Water]],
}

local MagicVulnerability = {
	1490--[[Curse of the Elements]],
	58410--[[Master Poisoner]],
	116202--[[Aura of the Elements]],
	34889--[[Fire Breath]],
	24844--[[Lightning Breath]],
}

a.spells["Pet Attack"] = {
	Override = function(z)
		if x.NoCC and x.PetAlive and x.Enemy and not x.PetActiveEnemy then
			s.FlashPet(PET_ACTION_ATTACK, "Green")
		elseif x.NoCC and x.PetAlive and x.Enemy and x.PetActiveEnemy and not s.SameTargetAsPet() then
			s.FlashPet(PET_ACTION_ATTACK, "Pink")
		end
	end,
}
a.spells["Pet Assist"] = {
	Override = function(z)
		if x.ActiveEnemy and x.PetAlive and not x.PetActiveEnemy and not a.PetAssistActive() then
			s.FlashPet(PET_MODE_ASSIST, "Pink")
		end
	end,
}

a.spells["Dark Intent"] = {
	ID = 109773--[[Dark Intent]],
	BuffUnit = "raid",
	Buff = SpellPowerMultiplier,
	NotWhileMoving = false,
	EarlyRefresh = 10,
	NoRangeCheck = 1,
	FlashColor = "Green",
}

a.spells["Curse of the Elements"] = {
	ID = 1490--[[Curse of the Elements]],
	Debuff = MagicVulnerability,
	EarlyRefresh = 5,
	NotWhileMoving = false,
	EnemyTargetNeeded = true,
	Check = function(z)
		return x.NoCC and not x.Metamorphosis
	end,
	Continue = true,
}

a.spells["Aura of the Elements"] = {
	ID = 1490--[[Curse of the Elements]],
	Buff = 116202--[[Aura of the Elements]],
	BuffUnit = "player",
	NotWhileMoving = false,
	Check = function(z)
		return x.NoCC and x.Metamorphosis
	end,
	FlashColor = "Blue",
	Continue = true,
}

a.spells["Summon Felhunter ooc"] = {
	ID = 691--[[Summon Felhunter]],
	NotWhileMoving = true,
	Check = function(z)
		return x.ShouldPermanentBuff and not x.PetAlive and not s.Buff({132614--[[Grimoire of Sacrifice]], 132612--[[Grimoire of Sacrifice]]}, "player")
	end,
	FlashID = {691--[[Summon Felhunter]], 112869--[[Summon Observer]], 10--[[Summon Demon]]}
}

a.spells["Summon Felguard ooc"] = {
	ID = 30146--[[Summon Felguard]],
	NotWhileMoving = true,
	Check = function(z)
		return x.ShouldPermanentBuff and not x.PetAlive and not s.Buff(132613--[[Grimoire of Sacrifice]], "player")
	end,
	FlashID = {30146--[[Summon Felguard]], 112870--[[Summon Wrathguard]], 10--[[Summon Demon]]}
}

a.spells["Dark Soul: Misery"] = {
	ID = 113860--[[Dark Soul: Misery]],
	MyBuff = 113860--[[Dark Soul: Misery]],
	BuffUnit = "player",
	NotWhileMoving = false,
	Check = function(z)
		return x.ShouldTemporaryBuff
	end,
	FlashColor = "Pink",
	NoGCD = true,
	Continue = true,
}

a.spells["Dark Soul: Knowledge"] = {
	ID = 113861--[[Dark Soul: Knowledge]],
	MyBuff = 113861--[[Dark Soul: Knowledge]],
	BuffUnit = "player",
	NotWhileMoving = false,
	Check = function(z)
		return x.ShouldTemporaryBuff and x.DemonicFury >= 750
	end,
	FlashColor = "Pink",
	NoGCD = true,
	Continue = true,
}

a.spells["Dark Soul: Instability"] = {
	ID = 113858--[[Dark Soul: Instability]],
	MyBuff = 113858--[[Dark Soul: Instability]],
	BuffUnit = "player",
	NotWhileMoving = false,
	Check = function(z)
		return x.ShouldTemporaryBuff
	end,
	FlashColor = "Pink",
	NoGCD = true,
	Continue = true,
}

a.spells["Grimoire: Felhunter"] = {
	ID = 108501--[[Grimoire of Service]],
	Check = function(z)
		return x.ActiveEnemy and s.HasTalent(108501--[[Grimoire of Service]])
	end,
	FlashID = {111897--[[Grimoire: Felhunter]], 108501--[[Grimoire of Service]]},
}

a.spells["Grimoire: Felguard"] = {
	ID = 111898--[[Grimoire: Felguard]],
	Check = function(z)
		return x.ActiveEnemy and s.HasTalent(108501--[[Grimoire of Service]])
	end,
	FlashID = {111898--[[Grimoire: Felguard]], 108501--[[Grimoire of Service]]},
}

a.spells["Grimoire of Sacrifice"] = {
	ID = 108503--[[Grimoire of Sacrifice]],
	MyBuff = {132612--[[Grimoire of Sacrifice]], 132613--[[Grimoire of Sacrifice]], 132614--[[Grimoire of Sacrifice]]},
	BuffUnit = "player",
	Check = function(z)
		return x.PetAlive and x.ActiveEnemy and s.HasTalent(108503--[[Grimoire of Sacrifice]])
	end,
	FlashColor = "Yellow",
	NoGCD = true,
}

a.spells["Summon Felhunter (Sacrifice down)"] = {
	ID = 691--[[Summon Felhunter]],
	NotWhileMoving = true,
	Check = function(z)
		return x.ActiveEnemy and not x.PetAlive and s.HasTalent(108503--[[Grimoire of Sacrifice]]) and not s.Buff(108503--[[Grimoire of Sacrifice]], "player")
	end,
	FlashID = {691--[[Summon Felhunter]], 10--[[Summon Demon]]}
}

a.spells["Summon Felguard (Sacrifice down)"] = {
	ID = 30146--[[Summon Felguard]],
	NotWhileMoving = true,
	Check = function(z)
		return x.ActiveEnemy and not x.PetAlive and s.HasTalent(108503--[[Grimoire of Sacrifice]]) and not s.Buff(108503--[[Grimoire of Sacrifice]], "player")
	end,
	FlashID = {30146--[[Summon Felguard]], 10--[[Summon Demon]]}
}

a.spells["Summon Doomguard"] = {
	ID = 18540--[[Summon Doomguard]],
	Check = function(z)
		return x.ActiveEnemy
	end,
	FlashColor = "Purple",
	NotWhileMoving = false,
	EnemyTargetNeeded = true,
	Continue = true,
}

a.spells["Summon Terrorguard"] = {
	ID = 112927--[[Summon Terrorguard]],
	Check = function(z)
		return x.ActiveEnemy
	end,
	FlashColor = "Purple",
	NotWhileMoving = false,
	EnemyTargetNeeded = true,
	Continue = true,
}

a.spells["Soul Swap with Soulburn"] = {
	ID = 86121--[[Soul Swap]],
	MyDebuff1 = 980--[[Agony]],
	MyDebuff2 = 172--[[Corruption]],
	MyDebuff3 = 30108--[[Unstable Affliction]],
	RequireMyBuff = 74434--[[Soulburn]],
	RequireBuffUnit = "player",
	ConsumeMyBuff = 74434--[[Soulburn]],
	ConsumeBuffUnit = "player",
	NotWhileMoving = false,
	Check = function(z)
		return x.NoCC
	end,
	RunLast = function(z)
		x.LastSoulburn = GetTime()
	end,
	EnemyTargetNeeded = true,
}

a.spells["Soulburn with Dark Soul"] = {
	ID = 74434--[[Soulburn]],
	Buff = 74434--[[Soulburn]],
	BuffUnit = "player",
	RequireMyBuff = 113860--[[Dark Soul: Misery]],
	RequireBuffUnit = "player",
	NotWhileMoving = false,
	Check = function(z)
		return x.ActiveEnemy and (GetTime() - x.LastSoulburn) > 20
	end,
	EnemyTargetNeeded = true,
	FlashColor = "Yellow",
	NoGCD = true,
}

a.spells["Soulburn under 20%"] = {
	ID = 74434--[[Soulburn]],
	Buff = 74434--[[Soulburn]],
	BuffUnit = "player",
	NotWhileMoving = false,
	Check = function(z)
		return x.ActiveEnemy and s.HealthPercent("target") <= 20
	end,
	CheckLast = function(z)
		if x.HasPandemic then
			return s.MyDebuffDuration(30108--[[Unstable Affliction]], "target") < (x.UATicks * x.TickSpeed * 0.5) or s.MyDebuffDuration(172--[[Corruption]], "target") < (x.CorruptionTicks * x.TickSpeed * 0.5) or s.MyDebuffDuration(980--[[Agony]], "target") < (x.AgonyTicks * x.TickSpeed * 0.5)
		else
			return s.MyDebuffDuration(30108--[[Unstable Affliction]], "target") < x.TickSpeed or s.MyDebuffDuration(172--[[Corruption]], "target") < x.TickSpeed or s.MyDebuffDuration(980--[[Agony]], "target") < x.TickSpeed
		end
	end,
	EnemyTargetNeeded = true,
	FlashColor = "Yellow",
	NoGCD = true,
}

a.spells["Haunt"] = {
	ID = 48181--[[Haunt]],
	MyDebuff = 48181--[[Haunt]],
	NotWhileMoving = true,
	Check = function(z)
		z.EarlyRefresh = (s.LastSpellTravelTime(48181--[[Haunt]], "target") or 0) + x.TickSpeed + s.CastTime(48181--[[Haunt]])
		return x.NoCC and not s.SpellOrAuraDelay(48181--[[Haunt]], "target", true)
	end,
	EnemyTargetNeeded = true,
}

a.spells["Agony refresh early if not channeling"] = {
	ID = 980--[[Agony]],
	MyDebuff = 980--[[Agony]],
	NotWhileMoving = false,
	Check = function(z)
		return x.NoCC and x.HasPandemic and x.TimeToDie >= 8
	end,
	CheckLast = function(z)
		z.EarlyRefresh = x.AgonyTicks * x.TickSpeed * 0.5
		return not s.Channeling(nil, "player") 
	end,
	EnemyTargetNeeded = true,
}

a.spells["Agony"] = {
	ID = 980--[[Agony]],
	MyDebuff = 980--[[Agony]],
	NotWhileMoving = false,
	Check = function(z)
		z.EarlyRefresh = x.TickSpeed
		return x.NoCC and x.TimeToDie >= 8
	end,
	EnemyTargetNeeded = true,
}

a.spells["Corruption refresh early if not channeling"] = {
	ID = 172--[[Corruption]],
	MyDebuff = 172--[[Corruption]],
	NotWhileMoving = false,
	Check = function(z)
		return x.NoCC and x.HasPandemic and x.TimeToDie >= 6
	end,
	CheckLast = function(z)
		z.EarlyRefresh = x.CorruptionTicks * x.TickSpeed * 0.5 
		return not s.Channeling(nil, "player")
	end,
	EnemyTargetNeeded = true,
}

a.spells["Corruption"] = {
	ID = 172--[[Corruption]],
	MyDebuff = 172--[[Corruption]],
	NotWhileMoving = false,
	Check = function(z)
		z.EarlyRefresh = x.TickSpeed
		return x.NoCC and x.TimeToDie >= 6
	end,
	EnemyTargetNeeded = true,
}

a.spells["Corruption (Demo)"] = {
	ID = 172--[[Corruption]],
	MyDebuff = 172--[[Corruption]],
	NotWhileMoving = false,
	Check = function(z)
		z.EarlyRefresh = x.CorruptionSpeed
		return x.NoCC and x.TimeToDie >= 6 and not x.Metamorphosis
	end,
	EnemyTargetNeeded = true,
}

a.spells["Unstable Affliction refresh early if not channeling"] = {
	ID = 30108--[[Unstable Affliction]],
	MyDebuff = 30108--[[Unstable Affliction]],
	NotWhileMoving = true,
	Check = function(z)
		return x.NoCC and x.HasPandemic and x.TimeToDie >= 5
	end,
	CheckLast = function(z)
		z.EarlyRefresh = x.UATicks * x.TickSpeed * 0.5 
		return not s.Channeling(nil, "player")
	end,
	EnemyTargetNeeded = true,
}

a.spells["Unstable Affliction"] = {
	ID = 30108--[[Unstable Affliction]],
	MyDebuff = 30108--[[Unstable Affliction]],
	NotWhileMoving = true,
	Check = function(z)
		z.EarlyRefresh = x.TickSpeed
		return x.NoCC and x.TimeToDie >= 5
	end,
	EnemyTargetNeeded = true,
}

a.spells["Shadowburn"] = {
	ID = 17877--[[Shadowburn]],
	Check = function(z)
		return x.NoCC
	end,
	FlashColor = "Orange",
	EnemyTargetNeeded = true,
}

a.spells["Chaos Bolt early"] = {
	ID = 116858--[[Chaos Bolt]],
	Check = function(z)
		return x.NoCC and (s.BuffStack(117828--[[Backdraft]], "player") < 3 or not x.HasPandemic) and (UnitPower("player", SPELL_POWER_BURNING_EMBERS, true) > (UnitPowerMax("player", SPELL_POWER_BURNING_EMBERS, true) - 5) or s.BuffDuration(113858--[[Dark Soul: Instability]], "player") > s.CastTime(116858--[[Chaos Bolt]])) --and s.PowerPercent("player", SPELL_POWER_MANA) <= 80
	end,
	NotWhileMoving = true,
	EnemyTargetNeeded = true,
}

a.spells["Chaos Bolt as fallback"] = {
	ID = 116858--[[Chaos Bolt]],
	Check = function(z)
		return x.NoCC and UnitPower("player", SPELL_POWER_BURNING_EMBERS, true) > (UnitPowerMax("player", SPELL_POWER_BURNING_EMBERS, true) - 20) --and s.PowerPercent("player", SPELL_POWER_MANA) < 10
	end,
	NotWhileMoving = true,
	EnemyTargetNeeded = true,
}

a.spells["Conflagrate"] = {
	ID = 17962--[[Conflagrate]],
	MyBuff = 117828--[[Backdraft]],
	StackGiven = 3,
	BuffUnit = "player",
	Check = function(z)
		return x.NoCC
	end,
	NotWhileMoving = false,
	EnemyTargetNeeded = true,
}

a.spells["Immolate"] = {
	ID = 348--[[Immolate]],
	MyDebuff = 348--[[Immolate]],
	Check = function(z)
		return x.NoCC and x.TimeToDie >= 5
	end,
	RunLast = function(z)
		z.EarlyRefresh = s.If(x.HasPandemic, x.ImmolateSpeed * x.ImmolateTicks, x.ImmolateSpeed)
	end,
	NotWhileMoving = true,
	EnemyTargetNeeded = true,
}

a.spells["Incinerate"] = {
	ID = 29722--[[Incinerate]],
	Check = function(z)
		return x.NoCC
	end,
	NotWhileMoving = true,
	EnemyTargetNeeded = true,
}

a.spells["Drain Soul as finisher"] = {
	ID = 1120--[[Drain Soul]],
	NotWhileMoving = true,
	Check = function(z)
		return x.NoCC
	end,
	CheckLast = function(z)
		return s.HealthPercent("target") <= 20
	end,
	EnemyTargetNeeded = true,
	FlashColor = "Orange",
}

a.spells["Life Tap if mana < 35%"] = {
	ID = 1454--[[Life Tap]],
	NotWhileMoving = false,
	Check = function(z)
		return x.ActiveEnemy
	end,
	CheckLast = function(z)
		return s.PowerPercent("player", SPELL_POWER_MANA) < 35 and not s.Channeling(nil, "player") 
	end,
}

a.spells["Life Tap while moving if mana < 80%"] = {
	ID = 1454--[[Life Tap]],
	NotWhileMoving = false,
	Check = function(z)
		return x.ActiveEnemy and s.Moving("player") and s.PowerPercent("player", SPELL_POWER_MANA) < 80
	end,
	CheckLast = function(z)
		return s.PowerPercent("player", SPELL_POWER_MANA) < s.HealthPercent("target")
	end,
}

a.spells["Life Tap if mana < 50%"] = {
	ID = 1454--[[Life Tap]],
	NotWhileMoving = false,
	Check = function(z)
		return x.ActiveEnemy
	end,
	CheckLast = function(z)
		return s.PowerPercent("player", SPELL_POWER_MANA) < 50
	end,
}

a.spells["Malefic Grasp"] = {
	ID = 103103--[[Malefic Grasp]],
	NotWhileMoving = true,
	Check = function(z)
		return x.NoCC
	end,
	EnemyTargetNeeded = true,
}

a.spells["Fel Flame while moving"] = {
	ID = 77799--[[Fel Flame]],
	NotWhileMoving = false,
	Check = function(z)
		return x.NoCC and s.Moving("player")
	end,
	EnemyTargetNeeded = true,
}

a.spells["Life Tap"] = {
	ID = 1454--[[Life Tap]],
	NotWhileMoving = false,
	Check = function(z)
		return x.ActiveEnemy
	end,
	NoStopChannel = true,
}

a.spells["Command Demon - Wrathstorm"] = {
	ID = 119915--[[Wrathstorm]],
	Override = function(z)
		return true
	end,
	NoGCD = true,
	FlashColor = "Yellow",
	FlashID = {119915--[[Wrathstorm]], 119898--[[Command Demon]]},
}

a.spells["Command Demon - Felstorm"] = {
	ID = 119914--[[Felstorm]],
	Override = function(z)
		return true
	end,
	NoGCD = true,
	FlashColor = "Yellow",
	FlashID = {119914--[[Felstorm]], 119898--[[Command Demon]]},
}

a.spells["Wrathstorm"] = {
	Type = "pet",
	ID = 115831--[[Wrathstorm]],
	Check = function(z)
		return x.PetAlive and x.PetActiveEnemy
	end,
	RunLast = function(z)
		a.Flash("Command Demon - Wrathstorm")
	end,
	FlashColor = "Yellow",
	Continue = true,
	NoGCD = true,
}

a.spells["Felstorm"] = {
	Type = "pet",
	ID = 89751--[[Felstorm]],
	Check = function(z)
		return x.PetAlive and x.PetActiveEnemy
	end,
	RunLast = function(z)
		a.Flash("Command Demon - Felstorm")
	end,
	FlashColor = "Yellow",
	Continue = true,
	NoGCD = true,
}

a.spells["Metamorphosis"] = {
	Type = "form",
	ID = 103958--[[Metamorphosis]],
	Buff = 103958--[[Metamorphosis]],
	BuffUnit = "player",
	NotIfActive = true,
	FlashBlink = true,
	Check = function(z)
		return x.ActiveEnemy
	end,
	CheckLast = function(z)
		return x.DarkSoul or (x.CorruptionRemaining > 2 and x.CorruptionRemaining < 5) or (x.DemonicFury >= min(900, x.TimeToDie * 30))
	end,
	FlashColor = "Yellow",
	Continue = true,
	NoGCD = true,
}

a.spells["Cancel Metamorphosis"] = {
	Type = "form",
	ID = 103958--[[Metamorphosis]],
	FlashBlink = true,
	Override = function(z)
		return x.ActiveEnemy and x.Metamorphosis and ((x.CorruptionRemaining > 20) or (x.CorruptionRemaining == 0)) and not x.DarkSoul and x.DemonicFury <= 750 and x.TimeToDie > 30
	end,
	FlashColor = "Pink",	
	NoGCD = true,
	Continue = true,
}

a.spells["Hand of Gul'dan"] = {
	ID = 105174--[[Hand of Gul'dan]],
	MyDebuff = 47960--[[Shadowflame]],
	Check = function(z)
		z.EarlyRefresh = (s.LastSpellTravelTime(105174--[[Hand of Gul'dan]], "target") or 0) + s.CastTime(686--[[Shadow Bolt]])
		return x.NoCC and not s.SpellOrAuraDelay(105174--[[Hand of Gul'dan]], "target", any)
	end,
	CheckLast = function(z)
		return not x.Metamorphosis
	end,
	EnemyTargetNeeded = true,
	Continue = true,
}

a.spells["Soul Fire"] = {
	ID = 6353--[[Soul Fire]],
	RequireMyBuff = 122355--[[Molten Core]],
	RequireBuffUnit = "player",
	ConsumeMyBuff = 122355--[[Molten Core]],
	ConsumeBuffUnit = "player",
	NotWhileMoving = true,
	Check = function(z)
		if s.HealthPercent("target") < 25 then
			z.FlashColor = "Orange"
		else
			z.FlashColor = "White"
		end
		return x.NoCC and (not x.Metamorphosis or x.DemonicFury > 160)
	end,
	EnemyTargetNeeded = true,
}

a.spells["Doom"] = {
	ID = 124913--[[Metamorphosis: Doom]],
	MyDebuff = 603--[[Doom]],
	RequireBuff = 103958--[[Metamorphosis]],
	RequireBuffUnit = "player",
	Check = function(z)
		if x.DarkSoul then
			z.EarlyRefresh = (x.DoomTicks - 2) * x.DoomSpeed
		else
			z.EarlyRefresh = s.If(x.HasPandemic, (x.DoomTicks * x.DoomSpeed * 0.5), x.DoomSpeed)
		end
		return x.NoCC and x.TimeToDie >= 30 
	end,
	FlashID = { 124913--[[Metamorphosis: Doom]], 603--[[Doom]], 172--[[Corruption]] },
	EnemyTargetNeeded = true,
}

a.spells["Touch of Chaos if Corruption < 20sec"] = {
	ID = 686--[[Shadow Bolt]],
	RequireBuff = 103958--[[Metamorphosis]],
	RequireBuffUnit = "player",
	Check = function(z)
		return x.NoCC and x.CorruptionRemaining > 0 and x.CorruptionRemaining < 20
	end,
	EnemyTargetNeeded = true,
	FlashID = { 112089--[[Metamorphosis: Touch of Chaos]], 686--[[Shadow Bolt]], 112092--[[Shadow Bolt]] },
}

a.spells["Touch of Chaos"] = {
	ID = 686--[[Shadow Bolt]],
	RequireBuff = 103958--[[Metamorphosis]],
	RequireBuffUnit = "player",
	Check = function(z)
		return x.NoCC
	end,
	EnemyTargetNeeded = true,
	FlashID = { 112089--[[Metamorphosis: Touch of Chaos]], 686--[[Shadow Bolt]], 112092--[[Shadow Bolt]] },
}

a.spells["Shadow Bolt"] = {
	ID = 686--[[Shadow Bolt]],
	Check = function(z)
		return x.NoCC
	end,
	CheckLast = function(z)
		return not x.Metamorphosis
	end,
	NotWhileMoving = true,
	EnemyTargetNeeded = true,
	FlashID = { 686--[[Shadow Bolt]], 112092--[[Shadow Bolt]] },
}

a.spells["Potion with Bloodlust or Boss HP <= 20"] = {
	Type = "item",
	ID = 76093--[[Potion of the Jade Serpent]],
	Buff = {105702--[[Potion of the Jade Serpent]], 79476--[[Volcanic Power]]},
	EnemyTargetNeeded = true,
	NoGCD = true,
	Check = function(z)
		return x.ShouldTemporaryBuff and x.ActiveEnemy and s.Boss()
	end,
	CheckLast = function(z)
		return a.Bloodlust() or (s.HealthPercent("target") <= 20)
	end,
	FlashID = {76093--[[Potion of the Jade Serpent]], 58091--[[Volcanic Potion]]},
	FlashColor = "Yellow",
	Continue = true,
}

a.spells["Carrion Swarm"] = {
	ID = 103967--[[Carrion Swarm]],
	Check = function(z)
		return x.NoCC
	end,
	CheckLast = function(z)
		return not s.Boss("target")
	end,
	Interrupt = true,
	FlashColor = "Aqua",
	EnemyTargetNeeded = true,
}

a.spells["Banish"] = {
	ID = 710--[[Banish]],
	EnemyTargetNeeded = 1,
	Check = function(z)
		return x.NoCC
	end,
	CheckLast = function(z)
		return not s.Boss("target")
	end,
	NotWhileMoving = true,
	FlashColor = "Pink",
}

a.spells["Create Healthstone"] = {
	ID = 6201--[[Create Healthstone]],
	Check = function(z)
		return x.ShouldTemporaryBuff 
	end,
	CheckLast = function(z)
		return GetItemCount(5512--[[Healthstone]], true) == 0 and not s.InGroup()
	end,
	NotWhileMoving = true,
	FlashColor = "Green",
}

a.spells["Create Soulwell"] = {
	ID = 29893--[[Create Soulwell]],
	Check = function(z)
		return x.ShouldTemporaryBuff 
	end,
	CheckLast = function(z)
		return GetItemCount(5512--[[Healthstone]], true) == 0 and s.InGroup()
	end,
	NotWhileMoving = true,
	FlashColor = "Green",
}

a.spells["Soulshatter"] = {
	ID = 29858--[[Soulshatter]],
	EnemyTargetNeeded = 1,
	FlashColor = "Red",
	Check = function(z)
		return x.ThreatPercent >= 80 and s.InGroup()
	end,
}

a.spells["Falling Meteor"] = {
	ID = 109151--[[Demonic Leap]],
	FlashColor = "Red",
	Check = function(z)
		return s.HasGlyph(56247--[[Glyph of Falling Meteor]]) and IsFalling()
	end,
}

a.spells["Curse of Exhaustion"] = {
	ID = 18223--[[Curse of Exhaustion]],
	CheckLast = function(z)
		return not s.Boss("target") and s.Moving("target") and s.EnemyTargetingYou("target")
	end,
	FlashColor = "Red",
	EnemyTargetNeeded = true,
}

a.spells["Dark Regeneration"] = {
	ID = 108359--[[Dark Regeneration]],
	CheckLast = function(z)
		return s.HasTalent(108359--[[Dark Regeneration]]) and s.HealthPercent("player") < 70
	end,
	NoGCD = true,
	FlashColor = "Red",
}

a.spells["Unbound Will"] = {
	ID = 108482--[[Unbound Will]],
	Check = function(z)
		return s.HasTalent(108482--[[Unbound Will]]) and (s.Debuff(nil, "player", nil, nil, 1) or s.MovementImpaired("player"))
	end,
	NoGCD = true,
	FlashColor = "Yellow",
}